local ADDON_NAME, ADDON_TABLE = ...

SLASH_CLA1 = "/cla"
CLA_LOOTLIST = {} -- String encoded NAME:GUID
CLA_SKINNING_TARGET_IDS = {
    11671, -- Core hound
    11673, -- Ancient Core Hound
}
CLA_PLAYERS_WITH_ADDON = {}
CLA_ITEM_TRACKING_TABLE = {} -- Aqual Quintessence
CLA_MESSAGE_COLOR = "96a9eb"
CLA_ADDON_PREFIX = "CLA"
CLA_DEBUG = false
CLA_AQUAL_QUINTESSENCE_ID = 17333
CLA_HOURGLASS_SAND_ID = 19183
CLA_CONFIG_DEFAULT = {
    showLoot = false
}
CLA_CONFIG_FRAME = nil
CLA_VERSION = "1.12"

function SlashCmdList.CLA(command)
    cla_print_debug('The /cla command was issued with parameter ' .. command)

    if command == "" then               CLA_OpenAddonSettings()
    elseif command == "help" then       CLA_PrintHelp()
    elseif command == "players" then    CLA_CreatePlayersListFrame()
    elseif command == "aq" then         CLA_PrintPlayersWithAQ()
    elseif command == "hs" then         CLA_PrintPlayersWithItem(CLA_HOURGLASS_SAND_ID)
    elseif command == "debug" then      CLA_ToggleDebug()
    elseif string.sub(command, 1, 7) == "inspect" then 
        local itemId = string.sub(command, 9)
        CLA_PrintPlayersWithItem(tonumber(itemId))
    else print("The command /cla " .. command .. " was not recognized")
    end
end

function CLA_OnEvent(self, event, ...)
    if event == "PLAYER_LOGIN" then
        C_ChatInfo.RegisterAddonMessagePrefix("CLA")
        cla_print_debug("The CLA prefix was registered")

        CLA_CreateOptionsFrame()

    elseif event == "CHAT_MSG_ADDON" then
        local prefix, text, channel, sender, target = ...

        sender = cla_get_player_name(sender)

        if prefix ~= "CLA" then return end

        if text == "DISCOVER" then
            C_ChatInfo.SendAddonMessage("CLA", "REPORT_VER:" .. CLA_VERSION, "WHISPER", sender)
            cla_print_debug("SendAddonMessage just sent a REPORT by whisper to " .. sender)

            return
        end

        if string.sub(text, 1, 10) == "REPORT_VER" then
            local addonVer = string.sub(text, 12)

            table.insert(CLA_PLAYERS_WITH_ADDON, {
                player = sender,
                version = addonVer
            })

            return
        end

        if text == "REPORT" then
            table.insert(CLA_PLAYERS_WITH_ADDON, {
                player = sender,
                version = "1.10"
            })

            return
        end

        if string.sub(text, 1, 13) == "DISCOVER_ITEM" then
            local itemId = string.sub(text, 15)
            local itemCount = GetItemCount(itemId, false)

            if itemCount > 0 then
                C_ChatInfo.SendAddonMessage("CLA", "REPORT_AQ", "WHISPER", sender)
            end

            return
        end

        if string.sub(text, 1, 11) == "REPORT_ITEM" then
            local itemId = string.sub(text, 13)

            if not cla_has_value(CLA_ITEM_TRACKING_TABLE, sender) then
                table.insert(CLA_ITEM_TRACKING_TABLE, sender)
            end

            return
        end

        if text == "DISCOVER_AQ" then
            local hasAQ = GetItemCount(CLA_AQUAL_QUINTESSENCE_ID, false)

            if hasAQ == 1 then
                C_ChatInfo.SendAddonMessage("CLA", "REPORT_AQ", "WHISPER", sender)
            end

            return
        end

        if text == "REPORT_AQ" then
            if not cla_has_value(CLA_ITEM_TRACKING_TABLE, sender) then
                table.insert(CLA_ITEM_TRACKING_TABLE, sender)
            end

            return
        end

        -- At this point it must be a name:destGuid message.
        cla_print_debug("A loot event was detected: " .. text)

        if CLA_LOOTLIST[text] ~= nil then
            CLA_LOOTLIST[text] = CLA_LOOTLIST[text] + 1
        else
            CLA_LOOTLIST[text] = 1

            cla_callback(1, function()
                local tableSplitResult = cla_mysplit(text, ':')
                local creatureName = tableSplitResult[1]
                local creatureGuid = tableSplitResult[2]
                local _, mlPlayerId = GetLootMethod();
                local playerIsMasterLooter = mlPlayerId == 0

                local _, _, _, _, _, npcId = strsplit("-", creatureGuid)

                cla_print_debug("Creature name for loot event is " .. creatureName)
                cla_print_debug("Creature GUID for loot event is " .. creatureGuid)
                cla_print_debug("Player is master looter: " .. cla_bool_to_string(playerIsMasterLooter))

                if CLA_LOOTLIST[text] > 1 then
                    if playerIsMasterLooter or ClassicLootAssistantConfig.showLoot then
                        cla_print_color("The " .. creatureName .. " that just died has loot on it!", CLA_MESSAGE_COLOR)
                    end
                    
                elseif CLA_LOOTLIST[text] == 1 then
                    if cla_has_value(CLA_SKINNING_TARGET_IDS, tonumber(npcId)) then
                        cla_print_color(sender .. " can loot the " .. creatureName, CLA_MESSAGE_COLOR)

                        if playerIsMasterLooter then
                            SendChatMessage("[Classic Loot Assistant]: You can loot the " .. creatureName .. " that just died", "WHISPER", nil, sender)
                        end
                    end
                end
            end)
        end
    elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
        local lootMethod, _, _ = GetLootMethod()

        if lootMethod == "master" or CLA_DEBUG then
            local time, eventName, _, _, _, _, _, destGuid, destName = CombatLogGetCurrentEventInfo()
            local creatureDied = string.sub(destGuid, 1, 8) == "Creature"

            if eventName == "UNIT_DIED" and creatureDied then
                cla_print_debug("A UNIT_DIED event was detected on creature " .. destName .. " with GUID " .. destGuid)

                cla_callback(1, function()
                    local hasLoot, _ = CanLootUnit(destGuid)

                    if CLA_DEBUG then
                        if hasLoot then
                            cla_print_debug("The " .. destName .. " that just died is lootable to you")
                        else
                            cla_print_debug("You can't loot the " .. destName .. " that just died")
                        end
                    end
    
                    if hasLoot then
                        cla_print_debug("Sending a loot event for creature " .. destName)
                        C_ChatInfo.SendAddonMessage("CLA", destName .. ":" .. destGuid, cla_get_channel())
                    end
                end)
            end
        end
    else
        print("This should never happen... Event = " .. event)
    end
end

function CLA_OpenAddonSettings()
    InterfaceOptionsFrame_OpenToCategory("Classic Loot Assistant")
    InterfaceOptionsFrame_OpenToCategory("Classic Loot Assistant")
    print("Use /cla help to get help on how to use the addon")
end

function CLA_PrintHelp()
    print("The following commands are available for Classic Loot Assistant")
    print("/cla -- Shows variable values")
    print("/cla players -- Shows the players with the addon enabled in your party/raid group")
    print("/cla aq -- Shows the players with the Aqual Quintessence in their inventory")
    print("/cla debug -- Toggles debug information")
end

-- Deprecated...
function CLA_PrintPlayersWithAQ()
    C_ChatInfo.SendAddonMessage("CLA", "DISCOVER_AQ", cla_get_channel())
    cla_print_debug('SendAddonMessage just sent DISCOVER message in channel ' .. cla_get_channel())

    cla_callback(1, function()
        local itemName, itemLink = GetItemInfo(CLA_AQUAL_QUINTESSENCE_ID)
        print("The following players has an " .. itemLink .. " in their inventory")
        table.foreach(CLA_ITEM_TRACKING_TABLE, print)
        CLA_ITEM_TRACKING_TABLE = {}
    end)
end

function CLA_PrintPlayersWithItem(itemId)
    C_ChatInfo.SendAddonMessage("CLA", "DISCOVER_ITEM:" .. itemId, cla_get_channel())
    cla_print_debug('SendAddonMessage just sent DISCOVER message in channel ' .. cla_get_channel())

    cla_callback(1, function()
        local itemName, itemLink = GetItemInfo(itemId)
        print("The following players has an " .. itemLink .. " in their inventory")
        table.foreach(CLA_ITEM_TRACKING_TABLE, print)
        CLA_ITEM_TRACKING_TABLE = {}
    end)
end

function CLA_ToggleDebug()
    if CLA_DEBUG then
        CLA_DEBUG = false
        print("CLA debug mode has been disabled")
    else
        CLA_DEBUG = true
        print("CLA debug mode has been enabled")
    end
end

function CLA_MakeCheckbox(name, parent)
    local cb = CreateFrame("CheckButton", name, parent, "UICheckButtonTemplate")
    cb:SetWidth(25)
    cb:SetHeight(25)
    cb:Show()

    local cblabel = cb:CreateFontString(nil, "OVERLAY")
    cblabel:SetFontObject("GameFontHighlight")
    cblabel:SetPoint("LEFT", cb,"RIGHT", 5,0)
    cb.label = cblabel
    return cb
end

function CLA_CreateOptionsFrame()
    local config = ClassicLootAssistantConfig or CLA_CONFIG_DEFAULT
    ClassicLootAssistantConfig = config

    local frame = CreateFrame("FRAME")
    frame.name = "Classic Loot Assistant"

    local label = frame:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
	label:SetPoint("TOPLEFT", 10, -15)
	label:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 10, -45)
	label:SetJustifyH("LEFT")
    label:SetJustifyV("TOP")
    label:SetText("Classic Loot Assistant")

    local content = CreateFrame("Frame", "CADOptionsContent", frame)
	content:SetPoint("TOPLEFT", 10, -10)
    content:SetPoint("BOTTOMRIGHT", -10, 10)

    local showLootCheckbox = CLA_MakeCheckbox(nil, content)
    showLootCheckbox.label:SetText("Always show loot messages")
    showLootCheckbox:SetPoint("TOPLEFT", 10, -60)
    content.hookTargetFrame = showLootCheckbox
    showLootCheckbox:SetScript("OnClick",function(self,button)
        f.Commands.hooktarget()
    end)

    local tooltipOnEnter = function(self, event)
        GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
        GameTooltip:SetText(self.tooltipText, nil, nil, nil, nil, false);
        GameTooltip:Show();
    end

    local tooltipOnLeave = function(self, event)
        GameTooltip:Hide();
    end

    showLootCheckbox.tooltipText = "Always show loot messages (usually only shown for the master looter)"
    showLootCheckbox:SetScript("OnEnter", tooltipOnEnter)
    showLootCheckbox:SetScript("OnLeave", tooltipOnLeave)

    showLootCheckbox:SetChecked(config.showLoot)

    showLootCheckbox:SetScript("OnClick",function(self,button)
        ClassicLootAssistantConfig.showLoot = not ClassicLootAssistantConfig.showLoot
    end)

    InterfaceOptions_AddCategory(frame)
    CLA_CONFIG_FRAME = frame
end

function CLA_CreatePlayersListFrame()
    local playerName, _ = UnitName("player")

    table.insert(CLA_PLAYERS_WITH_ADDON, {
        player = playerName,
        version = CLA_VERSION
    })

    C_ChatInfo.SendAddonMessage("CLA", "DISCOVER", cla_get_channel())
    cla_print_debug('SendAddonMessage just sent DISCOVER message in channel ' .. cla_get_channel())

    cla_callback(1, function()
        local cols = {
            {
                ["name"] = "Name",
                ["width"] = 100,
            },
            {
                ["name"] = "Version",
                ["width"] = 75,
            },
        }
    
        local players = CLA_GetPlayersInGroup()
        local data = {}

        for i, val in pairs(players) do
            local playerData = cla_table_find_player(CLA_PLAYERS_WITH_ADDON, val)

            if playerData ~= nil then
                if type(playerData) == "string" then
                    table.insert(data, {
                        playerData,
                        "1.00"
                    })
                else
                    table.insert(data, {
                        playerData.player,
                        playerData.version
                    })
                end
            else
                table.insert(data, {
                    val,
                    "MISSING"
                })
            end
        end

        cla_create_frame_with_table("PlayersTable", cols, data)

        CLA_PLAYERS_WITH_ADDON = {}
    end)
end

function CLA_GetPlayersInGroup()
    local plist={}

    if IsInRaid() then
        for i=1,40 do
            local pName, _ = UnitName('raid' .. i)

            if pName then
                tinsert(plist, pName)
            end
        end
    elseif IsInGroup() then
        for i=1,4 do
            local pName, _ = UnitName('party' .. i)

            if pName then
                tinsert(plist, pName)
            end
        end
    end

    if IsInGroup() or IsInRaid() == false then
        local pName, _ = UnitName('player')
        tinsert(plist, pName)
    end

    return plist
end
